"use strict";
var parser_1 = require('./parser');
var lodash_1 = require('lodash');
var store_1 = require('./store');
var readFromStore_1 = require('./readFromStore');
var diffAgainstStore_1 = require('./diffAgainstStore');
var queryPrinting_1 = require('./queryPrinting');
var QueryManager = (function () {
    function QueryManager(_a) {
        var _this = this;
        var networkInterface = _a.networkInterface, store = _a.store;
        this.idCounter = 0;
        this.networkInterface = networkInterface;
        this.store = store;
        this.selectionSetMap = {};
        this.resultCallbacks = {};
        this.store.subscribe(function () {
            _this.broadcastNewStore(_this.store.getState());
        });
    }
    QueryManager.prototype.mutate = function (_a) {
        var _this = this;
        var mutation = _a.mutation, variables = _a.variables;
        var mutationDef = parser_1.parseMutation(mutation);
        var request = {
            query: mutation,
            variables: variables,
        };
        return this.networkInterface.query(request)
            .then(function (result) {
            var resultWithDataId = lodash_1.assign({
                __data_id: 'ROOT_MUTATION',
            }, result.data);
            _this.store.dispatch(store_1.createQueryResultAction({
                result: resultWithDataId,
                selectionSet: mutationDef.selectionSet,
                variables: variables,
            }));
            return result.data;
        });
    };
    QueryManager.prototype.watchQuery = function (_a) {
        var _this = this;
        var query = _a.query, variables = _a.variables, _b = _a.forceFetch, forceFetch = _b === void 0 ? true : _b, _c = _a.returnPartialData, returnPartialData = _c === void 0 ? false : _c;
        var queryDef = parser_1.parseQuery(query);
        var watchHandle = this.watchSelectionSet({
            selectionSet: queryDef.selectionSet,
            rootId: 'ROOT_QUERY',
            typeName: 'Query',
            variables: variables,
        });
        var queryId = watchHandle.id;
        var queryDefForRequest = queryDef;
        var existingData;
        var request = {
            query: query,
            variables: variables,
        };
        if (!forceFetch) {
            var diffResult = this.diffRequestAgainstStore(request, queryDef.selectionSet);
            queryDefForRequest = diffResult.minimalQueryDef;
            existingData = diffResult.existingData;
            request = diffResult.minimalRequest;
        }
        if (request) {
            this.makeRequest(queryId, request, queryDefForRequest.selectionSet);
            if (returnPartialData) {
                setTimeout(function () {
                    throw new Error('partial result return not implemented');
                }, 0);
            }
        }
        else {
            setTimeout(function () {
                _this.broadcastQueryChange(queryId, existingData);
            }, 0);
        }
        return watchHandle;
    };
    QueryManager.prototype.broadcastNewStore = function (store) {
        var _this = this;
        lodash_1.forOwn(this.selectionSetMap, function (selectionSetWithRoot, queryId) {
            var resultFromStore = readFromStore_1.readSelectionSetFromStore({
                store: store,
                rootId: selectionSetWithRoot.rootId,
                selectionSet: selectionSetWithRoot.selectionSet,
                variables: selectionSetWithRoot.variables,
            });
            _this.broadcastQueryChange(queryId, resultFromStore);
        });
    };
    QueryManager.prototype.watchSelectionSet = function (selectionSetWithRoot) {
        var _this = this;
        var queryId = this.idCounter.toString();
        this.idCounter++;
        this.selectionSetMap[queryId] = selectionSetWithRoot;
        var isStopped = function () {
            return !_this.selectionSetMap[queryId];
        };
        return {
            id: queryId,
            isStopped: isStopped,
            stop: function () {
                _this.stopQuery(queryId);
            },
            onResult: function (callback) {
                if (isStopped()) {
                    throw new Error('Query was stopped. Please create a new one.');
                }
                _this.registerResultCallback(queryId, callback);
            },
        };
    };
    QueryManager.prototype.stopQuery = function (queryId) {
        delete this.selectionSetMap[queryId];
        delete this.registerResultCallback[queryId];
    };
    QueryManager.prototype.broadcastQueryChange = function (queryId, result) {
        this.resultCallbacks[queryId].forEach(function (callback) {
            callback(null, result);
        });
    };
    QueryManager.prototype.handleQueryErrorsAndStop = function (queryId, errors) {
        var errorCallbacks = this.resultCallbacks[queryId];
        this.stopQuery(queryId);
        if (errorCallbacks && errorCallbacks.length) {
            errorCallbacks.forEach(function (callback) {
                callback(errors);
            });
        }
        else {
            throw new Error('Uncaught query errors. Use onError on the query handle to get errors.');
        }
    };
    QueryManager.prototype.registerResultCallback = function (queryId, callback) {
        if (!this.resultCallbacks[queryId]) {
            this.resultCallbacks[queryId] = [];
        }
        this.resultCallbacks[queryId].push(callback);
    };
    QueryManager.prototype.makeRequest = function (queryId, request, selectionSet) {
        var _this = this;
        this.networkInterface.query(request)
            .then(function (result) {
            var errors = result.errors;
            if (errors && errors.length) {
                _this.handleQueryErrorsAndStop(queryId, errors);
            }
            else {
                var resultWithDataId = lodash_1.assign({
                    __data_id: 'ROOT_QUERY',
                }, result.data);
                _this.store.dispatch(store_1.createQueryResultAction({
                    result: resultWithDataId,
                    variables: request.variables,
                    selectionSet: selectionSet,
                }));
            }
        }).catch(function (errors) {
            _this.handleQueryErrorsAndStop(queryId, errors);
        });
    };
    QueryManager.prototype.diffRequestAgainstStore = function (request, selectionSet) {
        var clonedRequest = lodash_1.assign({}, request);
        var _a = diffAgainstStore_1.diffSelectionSetAgainstStore({
            selectionSet: selectionSet,
            variables: request.variables,
            rootId: 'ROOT_QUERY',
            store: this.store.getState(),
            throwOnMissingField: false,
        }), missingSelectionSets = _a.missingSelectionSets, result = _a.result;
        var queryDefForRequest;
        if (missingSelectionSets.length) {
            queryDefForRequest = queryPrinting_1.queryDefinition(missingSelectionSets);
            clonedRequest.query = queryPrinting_1.printQueryFromDefinition(queryDefForRequest);
        }
        else {
            clonedRequest = null;
        }
        return {
            existingData: result,
            minimalRequest: clonedRequest,
            minimalQueryDef: queryDefForRequest,
        };
    };
    return QueryManager;
}());
exports.QueryManager = QueryManager;
//# sourceMappingURL=QueryManager.js.map