"use strict";
var QueryManager_1 = require('../src/QueryManager');
var store_1 = require('../src/store');
var parser_1 = require('../src/parser');
var chai_1 = require('chai');
var graphql_1 = require('graphql');
var async_1 = require('async');
describe('QueryManager', function () {
    it('works with one query', function (done) {
        var queryManager = new QueryManager_1.QueryManager({
            networkInterface: {},
            store: store_1.createApolloStore(),
        });
        var fragmentDef = parser_1.parseFragment("\n      fragment FragmentName on Item {\n        id\n        stringField\n        numberField\n        nullField\n      }\n    ");
        var result = {
            id: 'abcd',
            stringField: 'This is a string!',
            numberField: 5,
            nullField: null,
        };
        var handle = queryManager.watchSelectionSet({
            rootId: 'abcd',
            typeName: 'Person',
            selectionSet: fragmentDef.selectionSet,
            variables: {},
        });
        handle.onResult(function (error, res) {
            chai_1.assert.deepEqual(res, result);
            done();
        });
        var store = {
            abcd: result,
        };
        queryManager.broadcastNewStore(store);
    });
    it('works with two queries', function (done) {
        var queryManager = new QueryManager_1.QueryManager({
            networkInterface: {},
            store: store_1.createApolloStore(),
        });
        var fragment1Def = parser_1.parseFragment("\n      fragment FragmentName on Item {\n        id\n        numberField\n        nullField\n      }\n    ");
        var fragment2Def = parser_1.parseFragment("\n      fragment FragmentName on Item {\n        id\n        stringField\n        nullField\n      }\n    ");
        var handle1 = queryManager.watchSelectionSet({
            rootId: 'abcd',
            typeName: 'Person',
            selectionSet: fragment1Def.selectionSet,
            variables: {},
        });
        var handle2 = queryManager.watchSelectionSet({
            rootId: 'abcd',
            typeName: 'Person',
            selectionSet: fragment2Def.selectionSet,
            variables: {},
        });
        var numDone = 0;
        handle1.onResult(function (error, res) {
            chai_1.assert.deepEqual(res, {
                id: 'abcd',
                numberField: 5,
                nullField: null,
            });
            numDone++;
            if (numDone === 2) {
                done();
            }
        });
        handle2.onResult(function (error, res) {
            chai_1.assert.deepEqual(res, {
                id: 'abcd',
                stringField: 'This is a string!',
                nullField: null,
            });
            numDone++;
            if (numDone === 2) {
                done();
            }
        });
        var store = {
            abcd: {
                id: 'abcd',
                stringField: 'This is a string!',
                numberField: 5,
                nullField: null,
            },
        };
        queryManager.broadcastNewStore(store);
    });
    it('properly roundtrips through a Redux store', function (done) {
        var query = "\n      query people {\n        allPeople(first: 1) {\n          people {\n            name\n          }\n        }\n      }\n    ";
        var data = {
            allPeople: {
                people: [
                    {
                        name: 'Luke Skywalker',
                    },
                ],
            },
        };
        var networkInterface = mockNetworkInterface([
            {
                request: { query: query },
                result: { data: data },
            },
        ]);
        var queryManager = new QueryManager_1.QueryManager({
            networkInterface: networkInterface,
            store: store_1.createApolloStore(),
        });
        var handle = queryManager.watchQuery({
            query: query,
        });
        handle.onResult(function (error, result) {
            chai_1.assert.deepEqual(result, data);
            done();
        });
    });
    it('properly roundtrips through a Redux store with variables', function (done) {
        var query = "\n      query people($firstArg: Int) {\n        allPeople(first: $firstArg) {\n          people {\n            name\n          }\n        }\n      }\n    ";
        var variables = {
            firstArg: 1,
        };
        var data = {
            allPeople: {
                people: [
                    {
                        name: 'Luke Skywalker',
                    },
                ],
            },
        };
        var networkInterface = mockNetworkInterface([
            {
                request: { query: query, variables: variables },
                result: { data: data },
            },
        ]);
        var queryManager = new QueryManager_1.QueryManager({
            networkInterface: networkInterface,
            store: store_1.createApolloStore(),
        });
        var handle = queryManager.watchQuery({
            query: query,
            variables: variables,
        });
        handle.onResult(function (error, result) {
            chai_1.assert.deepEqual(result, data);
            done();
        });
    });
    it('handles GraphQL errors', function (done) {
        var query = "\n      query people {\n        allPeople(first: 1) {\n          people {\n            name\n          }\n        }\n      }\n    ";
        var networkInterface = mockNetworkInterface([
            {
                request: { query: query },
                result: {
                    errors: [
                        {
                            name: 'Name',
                            message: 'This is an error message.',
                        },
                    ],
                },
            },
        ]);
        var queryManager = new QueryManager_1.QueryManager({
            networkInterface: networkInterface,
            store: store_1.createApolloStore(),
        });
        var handle = queryManager.watchQuery({
            query: query,
        });
        handle.onResult(function (error) {
            chai_1.assert.equal(error[0].message, 'This is an error message.');
            chai_1.assert.throws(function () {
                handle.onResult(function (err) { return null; });
            }, /Query was stopped. Please create a new one./);
            done();
        });
    });
    it('runs a mutation', function (done) {
        var mutation = "\n      mutation makeListPrivate {\n        makeListPrivate(id: \"5\")\n      }\n    ";
        var data = {
            makeListPrivate: true,
        };
        var networkInterface = mockNetworkInterface([
            {
                request: { query: mutation },
                result: { data: data },
            },
        ]);
        var queryManager = new QueryManager_1.QueryManager({
            networkInterface: networkInterface,
            store: store_1.createApolloStore(),
        });
        queryManager.mutate({
            mutation: mutation,
        }).then(function (resultData) {
            chai_1.assert.deepEqual(resultData, data);
            done();
        }).catch(function (err) {
            throw err;
        });
    });
    it('runs a mutation with variables', function (done) {
        var mutation = "\n      mutation makeListPrivate($listId: ID!) {\n        makeListPrivate(id: $listId)\n      }\n    ";
        var variables = {
            listId: '1',
        };
        var data = {
            makeListPrivate: true,
        };
        var networkInterface = mockNetworkInterface([
            {
                request: { query: mutation, variables: variables },
                result: { data: data },
            },
        ]);
        var queryManager = new QueryManager_1.QueryManager({
            networkInterface: networkInterface,
            store: store_1.createApolloStore(),
        });
        queryManager.mutate({
            mutation: mutation,
            variables: variables,
        }).then(function (resultData) {
            chai_1.assert.deepEqual(resultData, data);
            done();
        }).catch(function (err) {
            throw err;
        });
    });
    it('runs a mutation and puts the result in the store', function (done) {
        var mutation = "\n      mutation makeListPrivate {\n        makeListPrivate(id: \"5\") {\n          id,\n          isPrivate,\n        }\n      }\n    ";
        var data = {
            makeListPrivate: {
                id: '5',
                isPrivate: true,
            },
        };
        var networkInterface = mockNetworkInterface([
            {
                request: { query: mutation },
                result: { data: data },
            },
        ]);
        var store = store_1.createApolloStore();
        var queryManager = new QueryManager_1.QueryManager({
            networkInterface: networkInterface,
            store: store,
        });
        queryManager.mutate({
            mutation: mutation,
        }).then(function (resultData) {
            chai_1.assert.deepEqual(resultData, data);
            chai_1.assert.deepEqual(store.getState()['5'], { id: '5', isPrivate: true });
            done();
        }).catch(function (err) {
            throw err;
        });
    });
    it('diffs queries', function (done) {
        testDiffing([
            {
                query: "\n          {\n            people_one(id: \"1\") {\n              __typename,\n              id,\n              name\n            }\n          }\n        ",
                diffedQuery: "\n          {\n            people_one(id: \"1\") {\n              __typename,\n              id,\n              name\n            }\n          }\n        ",
                diffedQueryResponse: {
                    people_one: {
                        __typename: 'Person',
                        id: 'lukeId',
                        name: 'Luke Skywalker',
                    },
                },
                fullResponse: {
                    people_one: {
                        __typename: 'Person',
                        id: 'lukeId',
                        name: 'Luke Skywalker',
                    },
                },
                variables: {},
            },
            {
                query: "\n          {\n            people_one(id: \"1\") {\n              name\n              age\n            }\n          }\n        ",
                diffedQuery: "\n          {\n            __node_0: node(id: \"lukeId\") {\n              id\n              ... on Person {\n                age\n              }\n            }\n          }\n        ",
                diffedQueryResponse: {
                    __node_0: {
                        id: 'lukeId',
                        age: 45,
                    },
                },
                fullResponse: {
                    people_one: {
                        name: 'Luke Skywalker',
                        age: 45,
                    },
                },
                variables: {},
            },
            {
                query: "\n          {\n            people_one(id: \"1\") {\n              id\n              name\n              age\n            }\n          }\n        ",
                diffedQuery: null,
                diffedQueryResponse: null,
                fullResponse: {
                    people_one: {
                        id: 'lukeId',
                        name: 'Luke Skywalker',
                        age: 45,
                    },
                },
                variables: {},
            },
        ], done);
    });
});
function mockNetworkInterface(requestResultArray) {
    var requestToResultMap = {};
    requestResultArray.forEach(function (_a) {
        var request = _a.request, result = _a.result;
        requestToResultMap[requestToKey(request)] = result;
    });
    var queryMock = function (request) {
        return new Promise(function (resolve, reject) {
            var resultData = requestToResultMap[requestToKey(request)];
            if (!resultData) {
                throw new Error("Passed request that wasn't mocked: " + requestToKey(request));
            }
            resolve(resultData);
        });
    };
    return {
        query: queryMock,
    };
}
function requestToKey(request) {
    var query = request.query && graphql_1.print(graphql_1.parse(request.query));
    return JSON.stringify({
        variables: request.variables,
        debugName: request.debugName,
        query: query,
    });
}
function testDiffing(queryArray, done) {
    var networkInterface = mockNetworkInterface(queryArray.map(function (_a) {
        var diffedQuery = _a.diffedQuery, diffedQueryResponse = _a.diffedQueryResponse, _b = _a.variables, variables = _b === void 0 ? {} : _b;
        return {
            request: { query: diffedQuery, variables: variables },
            result: { data: diffedQueryResponse },
        };
    }));
    var queryManager = new QueryManager_1.QueryManager({
        networkInterface: networkInterface,
        store: store_1.createApolloStore(),
    });
    var steps = queryArray.map(function (_a) {
        var query = _a.query, fullResponse = _a.fullResponse, variables = _a.variables;
        return function (cb) {
            var handle = queryManager.watchQuery({
                query: query,
                variables: variables,
                forceFetch: false,
            });
            handle.onResult(function (error, result) {
                chai_1.assert.deepEqual(result, fullResponse);
                cb();
                handle.stop();
            });
        };
    });
    async_1.series(steps, function (err, res) {
        if (err) {
            throw err;
        }
        done();
    });
}
//# sourceMappingURL=QueryManager.js.map