"use strict";
var chai = require('chai');
var assert = chai.assert;
var src_1 = require('../src');
var graphql_1 = require('graphql');
var redux_todomvc_1 = require('./fixtures/redux-todomvc');
var redux_1 = require('redux');
var networkInterface_1 = require('../src/networkInterface');
var chaiAsPromised = require('chai-as-promised');
chai.use(chaiAsPromised);
describe('client', function () {
    it('does not require any arugments', function () {
        var client = new src_1.ApolloClient();
        assert.isDefined(client.apolloStore);
    });
    it('can allow passing in a network interface', function () {
        var networkInterface = networkInterface_1.createNetworkInterface('swapi');
        var client = new src_1.ApolloClient({
            networkInterface: networkInterface,
        });
        assert.equal(client.networkInterface._uri, networkInterface._uri);
    });
    it('can allow passing in a store', function () {
        var apolloStore = redux_1.createStore(redux_todomvc_1.rootReducer);
        var client = new src_1.ApolloClient({
            apolloStore: apolloStore,
        });
        assert.deepEqual(client.apolloStore.getState(), apolloStore.getState());
    });
    it('should allow for a single query to take place', function (done) {
        var query = "\n      query people {\n        allPeople(first: 1) {\n          people {\n            name\n          }\n        }\n      }\n    ";
        var data = {
            allPeople: {
                people: [
                    {
                        name: 'Luke Skywalker',
                    },
                ],
            },
        };
        var networkInterface = mockNetworkInterface({
            request: { query: query },
            result: { data: data },
        });
        var client = new src_1.ApolloClient({
            networkInterface: networkInterface,
        });
        return client.query({ query: query })
            .then(function (result) {
            assert.deepEqual(result, { data: data });
            done();
        });
    });
    it('should return errors correctly for a single query', function (done) {
        var query = "\n      query people {\n        allPeople(first: 1) {\n          people {\n            name\n          }\n        }\n      }\n    ";
        var errors = [
            {
                name: 'test',
                message: 'Syntax Error GraphQL request (8:9) Expected Name, found EOF',
            },
        ];
        var networkInterface = mockNetworkInterface({
            request: { query: query },
            result: { errors: errors },
        });
        var client = new src_1.ApolloClient({
            networkInterface: networkInterface,
        });
        return client.query({ query: query })
            .then(function (result) {
            assert.deepEqual(result, { errors: errors });
            done();
        });
    });
    it('should allow for subscribing to a request', function (done) {
        var query = "\n      query people {\n        allPeople(first: 1) {\n          people {\n            name\n          }\n        }\n      }\n    ";
        var data = {
            allPeople: {
                people: [
                    {
                        name: 'Luke Skywalker',
                    },
                ],
            },
        };
        var networkInterface = mockNetworkInterface({
            request: { query: query },
            result: { data: data },
        });
        var client = new src_1.ApolloClient({
            networkInterface: networkInterface,
        });
        var handle = client.watchQuery({ query: query });
        handle.onResult(function (error, result) {
            assert.deepEqual(result, data);
            done();
        });
    });
});
function mockNetworkInterface(mockedRequest) {
    var requestToResultMap = {};
    var request = mockedRequest.request, result = mockedRequest.result;
    requestToResultMap[requestToKey(request)] = result;
    var queryMock = function (req) {
        return new Promise(function (resolve, reject) {
            var resultData = requestToResultMap[requestToKey(req)];
            if (!resultData) {
                throw new Error("Passed request that wasn't mocked: " + requestToKey(req));
            }
            resolve(resultData);
        });
    };
    return {
        query: queryMock,
    };
}
function requestToKey(request) {
    var query = request.query && graphql_1.print(graphql_1.parse(request.query));
    return JSON.stringify({
        variables: request.variables,
        query: query,
    });
}
//# sourceMappingURL=client.js.map