"use strict";
var chai = require('chai');
var chaiAsPromised = require('chai-as-promised');
var lodash_1 = require('lodash');
chai.use(chaiAsPromised);
var assert = chai.assert;
var networkInterface_1 = require('../src/networkInterface');
describe('network interface', function () {
    describe('creating a network interface', function () {
        it('should throw without an endpoint', function () {
            assert.throws(function () {
                networkInterface_1.createNetworkInterface(null);
            }, /A remote enpdoint is required for a network layer/);
        });
        it('should create an instance with a given uri', function () {
            var networkInterface = networkInterface_1.createNetworkInterface('/graphql');
            assert.equal(networkInterface._uri, '/graphql');
        });
        it('should allow for storing of custom options', function () {
            var customOpts = {
                headers: { 'Authorizaion': 'working' },
                credentials: 'include',
            };
            var networkInterface = networkInterface_1.createNetworkInterface('/graphql', customOpts);
            assert.deepEqual(networkInterface._opts, lodash_1.assign({}, customOpts));
        });
        it('should not mutate custom options', function () {
            var customOpts = {
                headers: ['Authorizaion', 'working'],
                credentials: 'include',
            };
            var originalOpts = lodash_1.assign({}, customOpts);
            var networkInterface = networkInterface_1.createNetworkInterface('/graphql', customOpts);
            delete customOpts.headers;
            assert.deepEqual(networkInterface._opts, originalOpts);
        });
    });
    describe('making a request', function () {
        it('should fetch remote data', function () {
            var swapi = networkInterface_1.createNetworkInterface('http://graphql-swapi.parseapp.com/');
            var simpleRequest = {
                query: "\n          query people {\n            allPeople(first: 1) {\n              people {\n                name\n              }\n            }\n          }\n        ",
                variables: {},
                debugName: 'People query',
            };
            return assert.eventually.deepEqual(swapi.query(simpleRequest), {
                data: {
                    allPeople: {
                        people: [
                            {
                                name: 'Luke Skywalker',
                            },
                        ],
                    },
                },
            });
        });
        it('should return errors if the server responds with them', function () {
            var swapi = networkInterface_1.createNetworkInterface('http://graphql-swapi.parseapp.com/');
            var simpleRequest = {
                query: "\n          query people {\n            allPeople(first: 1) {\n              people {\n                name\n              }\n          }\n        ",
                variables: {},
                debugName: 'People query',
            };
            return assert.eventually.deepEqual(swapi.query(simpleRequest), {
                errors: [
                    {
                        message: 'Syntax Error GraphQL request (8:9) Expected Name, found EOF\n\n7:           }\n8:         \n           ^\n',
                        locations: [
                            {
                                line: 8,
                                column: 9,
                            },
                        ],
                    },
                ],
            });
        });
        it('should throw on a network error', function () {
            var nowhere = networkInterface_1.createNetworkInterface('http://does-not-exist.parseapp.com/');
            var doomedToFail = {
                query: "\n          query people {\n            allPeople(first: 1) {\n              people {\n                name\n              }\n            }\n          }\n        ",
                variables: {},
                debugName: 'People Query',
            };
            return assert.isRejected(nowhere.query(doomedToFail));
        });
    });
});
//# sourceMappingURL=networkInterface.js.map